#include <common.h>

#include <command.h>
#include <movi.h>
#include <nand.h>

#include <regs.h>

#if defined(CONFIG_S3C2412) || defined(CONFIG_S3C2442)
#include "../cpu/s3c24xx/usbd-fs.h"
#elif defined(CONFIG_S3C2443) || defined(CONFIG_S3C2450) || defined(CONFIG_S3C2416)
#include "../cpu/s3c24xx/usbd-hs.h"
#elif defined(CONFIG_S3C6400) || defined(CONFIG_S3C6410) || defined(CONFIG_S3C6430)
#include "../cpu/s3c64xx/usbd-otg-hs.h"
#elif defined(CONFIG_S5PC100)
#include "../cpu/s5pc1xx/usbd-otg-hs.h"
#elif defined(CONFIG_S5P6440)
#include "../cpu/s5p64xx/usbd-otg-hs.h"
#else
#error "* CFG_ERROR : you have to setup right Samsung CPU configuration"
#endif


struct movi_offset_t ofsinfo;
extern nand_info_t nand_info[];


#define readl(addr) (*(volatile unsigned int*)(addr))
#define writel(b,addr) ((*(volatile unsigned int *) (addr)) = (b))

#define BOOTCOMMAND_ANDROID_BBP	"nand read.bbp 20008000 200000 300000;nand read.bbp 20800000 600000 100000;bootm 20008000 20800000"
#define BOOTCOMMAND_ANDROID_NR	"nand read 20008000 200000 300000;nand read 20800000 600000 100000;bootm 20008000 20800000"


#if 0
int do_testhsmmc(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	uint start_blk, blknum;
	start_blk = simple_strtoul(argv[1], NULL, 10);
	blknum = simple_strtoul(argv[2], NULL, 10);

	if(argc < 3)
		return 0;

	printf("COMMAND --> start_blk = 0x%x, blknum = %d\n", start_blk, blknum);
	test_hsmmc(4, 1, start_blk, blknum);
	return 1;
}

U_BOOT_CMD(
	testhsmmc,	4,	0,	do_testhsmmc,
	"testhsmmc - hsmmc testing write/read\n",
	NULL
);
#endif


#if !defined(CONFIG_SMDK6440)
int do_movi(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
{
	char *cmd;
	unsigned int addr, blk, blocks;
	uint last_blkpos, bytes;

	cmd = argv[1];

	if (strcmp(cmd, "init") == 0) {
		if (argc < 4)
			goto usage;
		
		last_blkpos = (uint) simple_strtoul(argv[2], NULL, 10);
		movi_hc = (uint) simple_strtoul(argv[3], NULL, 10);

		if (movi_set_ofs(last_blkpos))
			movi_init();
	} else {
		if (argc == 4 || (strcmp(argv[2], "rootfs") == 0))
			addr = (ulong) simple_strtoul(argv[3], NULL, 16);
		else if (argc == 5)
			addr = (ulong) simple_strtoul(argv[4], NULL, 16);
		else
			goto usage;

		if (addr >= 0xc0000000)
			addr = virt_to_phys(addr);

		if (ofsinfo.last == 0) {
			printf("### Execute 'movi init' command first!\n");
			return -1;
		}
		
		if (strcmp(cmd, "read") == 0) {
			if (argc == 4 || (strcmp(argv[2], "rootfs") == 0)) {
				if (strcmp(argv[2], "u-boot") == 0) {
					printf("Reading bootloader.. ");
					movi_read((uint) addr, ofsinfo.bl2, MOVI_BL2_BLKCNT);
					printf("completed\n");

				} else if (strcmp(argv[2], "kernel") == 0) {
					printf("Reading kernel from sector %d (%d sectors).. ", ofsinfo.kernel, MOVI_ZIMAGE_BLKCNT);
					movi_read((uint) addr, ofsinfo.kernel, MOVI_ZIMAGE_BLKCNT);
					printf("completed\n");

				} else if (strcmp(argv[2], "rootfs") == 0) {
					bytes = simple_strtoul(argv[4], NULL, 16);
					
					if (argc == 5)
						blocks = (ulong) (bytes / MOVI_BLKSIZE);
					else
						blocks = MOVI_ROOTFS_BLKCNT;

					if (bytes % MOVI_BLKSIZE > 0)
						blocks++;

					blk = ofsinfo.rootfs;

					printf("Reading rootfs from sector %d (%d sectors).. ", blk, blocks);

					while (blocks > 0) {
						if (blocks < MOVI_RW_MAXBLKS) {
							movi_read((uint) addr, blk, blocks);
							blocks = 0;
						} else {
							movi_read((uint) addr, blk, MOVI_RW_MAXBLKS);
							addr += (MOVI_RW_MAXBLKS * MOVI_BLKSIZE);
							blk += MOVI_RW_MAXBLKS;
							blocks -= MOVI_RW_MAXBLKS;
						}
					}
					printf("completed\n");
				} else
					goto usage;
			} else {
				blk = (uint) simple_strtoul(argv[2], NULL, 10);
				bytes = (uint) simple_strtoul(argv[3], NULL, 16);

				if (blk >= 0 && blk <= ofsinfo.last) {
					blocks = (int) (bytes / MOVI_BLKSIZE);

					if (bytes % MOVI_BLKSIZE > 0)
						blocks++;

					printf("Reading data from sector %d (%d sectors).. ", blk, blocks);

					while (blocks > 0) {
						if (blocks < MOVI_RW_MAXBLKS) {
							movi_read((uint) addr, blk, blocks);
							blocks = 0;
						} else {
							movi_read((uint) addr, blk, MOVI_RW_MAXBLKS);
							addr += (MOVI_RW_MAXBLKS * MOVI_BLKSIZE);
							blk += MOVI_RW_MAXBLKS;
							blocks -= MOVI_RW_MAXBLKS;
						}
					}
					printf("completed\n");
				} else
					goto usage;
			}
		} else if (strcmp(cmd, "write") == 0) {
			if (argc == 4 || (strcmp(argv[2], "rootfs") == 0)) {
				if (strcmp(argv[2], "u-boot") == 0) {
					printf("Writing 1st bootloader to sector %d (%d sectors).. ", ofsinfo.bl1, MOVI_BL1_BLKCNT);
					movi_write((uint) addr, ofsinfo.bl1, MOVI_BL1_BLKCNT);
					printf("completed\nWriting 2nd bootloader to sector %d (%d sectors).. ", ofsinfo.bl2, MOVI_BL2_BLKCNT);
					movi_write((uint) addr, ofsinfo.bl2, MOVI_BL2_BLKCNT);
					printf("completed\n");

				} else if (strcmp(argv[2], "kernel") == 0) {
					printf("Writing kernel to sector %d (%d sectors).. ", ofsinfo.kernel, MOVI_ZIMAGE_BLKCNT);
					movi_write((uint) addr, ofsinfo.kernel, MOVI_ZIMAGE_BLKCNT);
					printf("completed\n");

				} else if (strcmp(argv[2], "env") == 0) {
					printf("Writing env to sector %d (%d sectors).. ", ofsinfo.env, MOVI_ENV_BLKCNT);
					movi_write((uint) addr, ofsinfo.env, MOVI_ENV_BLKCNT);
					printf("completed\n");

				} else if (strcmp(argv[2], "rootfs") == 0) {
					blk = ofsinfo.rootfs;
					bytes = simple_strtoul(argv[4], NULL, 16);
					
					if (argc == 5)
						blocks = (ulong) (bytes / MOVI_BLKSIZE);
					else
						blocks = MOVI_ROOTFS_BLKCNT;

					if (bytes % MOVI_BLKSIZE > 0)
						blocks++;

					printf("Writing rootfs to sector %d (%d sectors).. ", blk, blocks);

					while (blocks > 0) {
						if (blocks < MOVI_RW_MAXBLKS) {
							movi_write((uint) addr, blk, blocks);
							blocks = 0;
						} else {
							movi_write((uint) addr, blk, MOVI_RW_MAXBLKS);
							addr += (MOVI_RW_MAXBLKS * MOVI_BLKSIZE);
							blk += MOVI_RW_MAXBLKS;
							blocks -= MOVI_RW_MAXBLKS;
						}
					}
					printf("completed\n");
				} else
					goto usage;
			} else {
				blk = (uint) simple_strtoul(argv[2], NULL, 10);
				bytes = (uint) simple_strtoul(argv[3], NULL, 16);

				if (blk >= 0 && blk <= ofsinfo.last) {
					blocks = (int) (bytes / MOVI_BLKSIZE);

					if (bytes % MOVI_BLKSIZE > 0)
						blocks++;

					printf("Writing data to sector %d (%d sectors).. ", blk, blocks);

					while (blocks > 0) {
						if (blocks < MOVI_RW_MAXBLKS) {
							movi_write((uint) addr, blk, blocks);
							blocks = 0;
						} else {
							movi_write((uint) addr, blk, MOVI_RW_MAXBLKS);
							addr += (MOVI_RW_MAXBLKS * MOVI_BLKSIZE);
							blk += MOVI_RW_MAXBLKS;
							blocks -= MOVI_RW_MAXBLKS;
						}
					}
					printf("completed\n");
				} else
					goto usage;
			}
		} else {
			goto usage;
		}
	}

	return 1;

usage:
	printf("Usage:\n%s\n", cmdtp->usage);
	return -1;
}
#else
int do_movi(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
{
	char *cmd;
	unsigned int addr, blk, blocks;
	uint last_blkpos, bytes;

	cmd = argv[1];

	if (strcmp(cmd, "init") == 0) {
		if (argc < 4)
			goto usage;
		
		last_blkpos = (uint) simple_strtoul(argv[2], NULL, 10);
		movi_hc = (uint) simple_strtoul(argv[3], NULL, 10);

		if (movi_set_ofs(last_blkpos))
			movi_init();
	} else {
		if (argc == 4 || (strcmp(argv[2], "rootfs") == 0))
			addr = (ulong) simple_strtoul(argv[3], NULL, 16);
		else if (argc == 5)
			addr = (ulong) simple_strtoul(argv[4], NULL, 16);
		else
			goto usage;

		if (addr >= 0xc0000000)
			addr = virt_to_phys(addr);

		if ((ofsinfo.last == 0) && (movi_emmc ==0)) {
			printf("### Execute 'movi init' command first!\n");
			return -1;
		}
		
		if (strcmp(cmd, "read") == 0) {
			if (argc == 4 || (strcmp(argv[2], "rootfs") == 0)) {
				if (strcmp(argv[2], "u-boot") == 0) {
					if (movi_emmc == 1) {	/* eMMC_4.3 */
						printf("eMMC Reading bootloader.. from sector %d (%d sectors)..", ofsinfo.bl2, EMMC_BL_BLKCNT);
						emmc_read((uint) addr, ofsinfo.bl2, EMMC_BL_BLKCNT);
					} else {
						printf("Reading bootloader.. ");
						movi_read((uint) addr, ofsinfo.bl2, MOVI_BL2_BLKCNT);
					}
					printf("completed\n");

				} else if (strcmp(argv[2], "kernel") == 0) {
					if (movi_emmc == 1) {	/* eMMC_4.3 */
						printf("eMMC Reading kernel from sector %d (%d sectors).. ", ofsinfo.kernel, EMMC_KERNEL_BLKCNT);
						emmc_read((uint) addr, ofsinfo.kernel, EMMC_KERNEL_BLKCNT);
					} else {
						printf("Reading kernel from sector %d (%d sectors).. ", ofsinfo.kernel, MOVI_ZIMAGE_BLKCNT);
						movi_read((uint) addr, ofsinfo.kernel, MOVI_ZIMAGE_BLKCNT);
					}
					printf("completed\n");

				} else if (strcmp(argv[2], "rootfs") == 0) {
					bytes = simple_strtoul(argv[4], NULL, 16);
					
					if (argc == 5)
						blocks = (ulong) (bytes / MOVI_BLKSIZE);
					else
						blocks = MOVI_ROOTFS_BLKCNT;

					if (bytes % MOVI_BLKSIZE > 0)
						blocks++;

					blk = ofsinfo.rootfs;

					if (movi_emmc == 1) {	/* eMMC_4.3 */
						printf("eMMC Reading rootfs from sector %d (%d sectors).. ", ofsinfo.rootfs, EMMC_ROOTFS_BLKCNT);
						emmc_read((uint) addr, ofsinfo.rootfs, EMMC_ROOTFS_BLKCNT);
					} else {
						printf("Reading rootfs from sector %d (%d sectors).. ", blk, blocks);

						while (blocks > 0) {
							if (blocks < MOVI_RW_MAXBLKS) {
								movi_read((uint) addr, blk, blocks);
								blocks = 0;
							} else {
								movi_read((uint) addr, blk, MOVI_RW_MAXBLKS);
								addr += (MOVI_RW_MAXBLKS * MOVI_BLKSIZE);
								blk += MOVI_RW_MAXBLKS;
								blocks -= MOVI_RW_MAXBLKS;
							}
						}
					}
					printf("completed\n");
				} else
					goto usage;
			} else {
				blk = (uint) simple_strtoul(argv[2], NULL, 10);
				bytes = (uint) simple_strtoul(argv[3], NULL, 16);

				if (blk >= 0 && blk <= ofsinfo.last) {
					blocks = (int) (bytes / MOVI_BLKSIZE);

					if (bytes % MOVI_BLKSIZE > 0)
						blocks++;

					if (movi_emmc == 1) {	/* eMMC_4.3 */
						if ((blk + blocks) <= ofsinfo.last ) {
							printf("eMMC Reading data from sector %d (%d sectors).. ", blk, blocks);
							emmc_read((uint) addr, blk, blocks);
						}
					} else {
						printf("Reading data from sector %d (%d sectors).. ", blk, blocks);

						while (blocks > 0) {
							if (blocks < MOVI_RW_MAXBLKS) {
								movi_read((uint) addr, blk, blocks);
								blocks = 0;
							} else {
								movi_read((uint) addr, blk, MOVI_RW_MAXBLKS);
								addr += (MOVI_RW_MAXBLKS * MOVI_BLKSIZE);
								blk += MOVI_RW_MAXBLKS;
								blocks -= MOVI_RW_MAXBLKS;
							}
						}
					}
					printf("completed\n");
				} else
					goto usage;
			}
		} else if (strcmp(cmd, "write") == 0) {
			if (argc == 4 || (strcmp(argv[2], "rootfs") == 0)) {
				if (strcmp(argv[2], "u-boot") == 0) {
					if (movi_emmc == 1) {	/* eMMC_4.3 */
						printf("eMMC Writing bootloader to sector %d (%d sectors).. ", ofsinfo.bl2, EMMC_BL_BLKCNT);
						emmc_write((uint) addr, ofsinfo.bl2, EMMC_BL_BLKCNT);
					} else {		/* SD/MMC */
						printf("Writing 1st bootloader to sector %d (%d sectors).. ", ofsinfo.bl1, MOVI_BL1_BLKCNT);
						movi_write((uint) addr, ofsinfo.bl1, MOVI_BL1_BLKCNT);
						printf("completed\nWriting 2nd bootloader to sector %d (%d sectors).. ", ofsinfo.bl2, MOVI_BL2_BLKCNT);
						movi_write((uint) addr, ofsinfo.bl2, MOVI_BL2_BLKCNT);
					}
					printf("completed\n");

				} else if (strcmp(argv[2], "kernel") == 0) {
					if (movi_emmc == 1) {	/* eMMC_4.3 */
						printf("eMMC Writing kernel to sector %d (%d sectors).. ", ofsinfo.kernel, EMMC_KERNEL_BLKCNT);
						emmc_write((uint) addr, ofsinfo.kernel, EMMC_KERNEL_BLKCNT);
					} else {		/* SD/MMC */
						printf("Writing kernel to sector %d (%d sectors).. ", ofsinfo.kernel, MOVI_ZIMAGE_BLKCNT);
						movi_write((uint) addr, ofsinfo.kernel, MOVI_ZIMAGE_BLKCNT);
					}
					printf("completed\n");

				} else if (strcmp(argv[2], "env") == 0) {
					if (movi_emmc == 1) {	/* eMMC_4.3 */
						printf("eMMC Writing env to sector %d (%d sectors).. ", ofsinfo.env, EMMC_ENV_BLKCNT);
						emmc_write((uint) addr, ofsinfo.env, EMMC_ENV_BLKCNT);
					} else {		/* SD/MMC */
						printf("Writing env to sector %d (%d sectors).. ", ofsinfo.env, MOVI_ENV_BLKCNT);
						movi_write((uint) addr, ofsinfo.env, MOVI_ENV_BLKCNT);
					}
					printf("completed\n");

				} else if (strcmp(argv[2], "rootfs") == 0) {
					if (movi_emmc == 1) {	/* eMMC_4.3 */
						printf("eMMC Writing rootfs to sector %d (%d sectors).. ", ofsinfo.rootfs, EMMC_ROOTFS_BLKCNT);
						emmc_write((uint) addr, ofsinfo.rootfs, EMMC_ROOTFS_BLKCNT);
					} else {		/* SD/MMC */
						blk = ofsinfo.rootfs;
						bytes = simple_strtoul(argv[4], NULL, 16);
						
						if (argc == 5)
							blocks = (ulong) (bytes / MOVI_BLKSIZE);
						else
							blocks = MOVI_ROOTFS_BLKCNT;

						if (bytes % MOVI_BLKSIZE > 0)
							blocks++;

						printf("Writing rootfs to sector %d (%d sectors).. ", blk, blocks);

						while (blocks > 0) {
							if (blocks < MOVI_RW_MAXBLKS) {
								movi_write((uint) addr, blk, blocks);
								blocks = 0;
							} else {
								movi_write((uint) addr, blk, MOVI_RW_MAXBLKS);
								addr += (MOVI_RW_MAXBLKS * MOVI_BLKSIZE);
								blk += MOVI_RW_MAXBLKS;
								blocks -= MOVI_RW_MAXBLKS;
							}
						}
					}
					printf("completed\n");
				} else
					goto usage;
			} else {
				blk = (uint) simple_strtoul(argv[2], NULL, 10);
				bytes = (uint) simple_strtoul(argv[3], NULL, 16);

				if (blk >= 0 && blk <= ofsinfo.last) {
					blocks = (int) (bytes / MOVI_BLKSIZE);

					if (bytes % MOVI_BLKSIZE > 0)
						blocks++;

					if (movi_emmc == 1) {	/* eMMC_4.3 */
						if ((blk + blocks) <= ofsinfo.last ) {
							printf("eMMC Writing data to sector %d (%d sectors).. ", blk, blocks);
							emmc_write((uint) addr, blk, blocks);
						}
					} else {		/* SD/MMC */
						printf("Writing data to sector %d (%d sectors).. ", blk, blocks);

						while (blocks > 0) {
							if (blocks < MOVI_RW_MAXBLKS) {
								movi_write((uint) addr, blk, blocks);
								blocks = 0;
							} else {
								movi_write((uint) addr, blk, MOVI_RW_MAXBLKS);
								addr += (MOVI_RW_MAXBLKS * MOVI_BLKSIZE);
								blk += MOVI_RW_MAXBLKS;
								blocks -= MOVI_RW_MAXBLKS;
							}
						}
					}
					printf("completed\n");
				} else
					goto usage;
			}
		} else {
			goto usage;
		}
	}

	return 1;

usage:
	printf("Usage:\n%s\n", cmdtp->usage);
	return -1;
}
#endif

U_BOOT_CMD(
	movi,	5,	0,	do_movi,
	"movi\t- moviNAND sub-system\n",
	"init {total sectors} {hc(0/1)} - Initialize moviNAND\n"
	"movi read  {u-boot | kernel} {addr} - Read data from moviNAND\n"
	"movi write {u-boot | kernel} {addr} - Write data to moviNAND\n"
	"movi read  rootfs {addr} [bytes(hex)] - Read rootfs data from moviNAND by size\n"
	"movi write rootfs {addr} [bytes(hex)] - Write rootfs data to moviNAND by size\n"
	"movi read  {sector#} {bytes(hex)} {addr} - Read data from moviNAND sector#\n"
	"movi write {sector#} {bytes(hex)} {addr} - Write data to moviNAND sector#\n"
);

#if defined(CONFIG_CMD_NAND)
int do_insnand(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
{
	size_t total = CFG_ENV_OFFSET;
	ulong addr = PHYS_SDRAM_1;

	movi_read((uint) addr, ofsinfo.bl2, MOVI_BL2_BLKCNT);
	nand_erase(&nand_info[0], 0x0, CFG_ENV_OFFSET + CFG_ENV_SIZE);
	nand_write(&nand_info[0], 0x0, &total, (u_char *) addr);

	printf("Done\n");

	return 1;
}


#define NORMAL_FS	0
#define YAFFS_FS	1

int ft_installImage(nand_write_options_t opts, int nFsType)
{
	int ret = 0;

	if(nFsType == NORMAL_FS)
	{
		opts.pad = 1;
		opts.writeoob = 0;
	}
	else
	{
		opts.pad = 0;
		opts.writeoob = 1;
	}

	opts.blockalign = 1;
	opts.autoplace = 1;	
	
//------------------------------------------------------------------
	if(opts.length > 0x500000)	opts.quiet  = 0;
	else				opts.quiet = 1;

	nand_erase(&nand_info[0], opts.buffer, &opts.length);
	ret = nand_write_opts(&nand_info[0], &opts);

	return ret == 0 ? 0 : 1;

	return 1;	
}




//int do_insAndroidUboot(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
int do_insAndroidUboot()
{
	int ret = 0;

	nand_write_options_t opts;
	memset(&opts, 0, sizeof(opts));

//------------------------------------------------------------------
	printf("Start android uboot installation. \n");
	
	opts.offset = 0x0;
	opts.length = readl(0x2000FFA8);
	opts.buffer = readl(0x2000FFA0);

	ret = ft_installImage(opts,NORMAL_FS);
	
	printf("Installed Android uboot image. \n");

	return ret == 0 ? 0 : 1;
	return 1;
}




//int do_insAndroidzImage(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
int do_insAndroidzImage()
{
	int ret = 0;

	nand_write_options_t opts;
	memset(&opts, 0, sizeof(opts));
//------------------------------------------------------------------
	printf("Start android zImage installation. \n");

	opts.offset = 0x200000;
	opts.length = readl(0x2000FFB8);
	opts.buffer = readl(0x2000FFB0);

	ret = ft_installImage(opts,NORMAL_FS);

	printf("Installed Android zImage image. \n");

	return ret == 0 ? 0 : 1;
	return 1;
}




//int do_insAndroidRamdisk(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
int do_insAndroidRamdisk()
{
	int ret = 0;

	nand_write_options_t opts;
	memset(&opts, 0, sizeof(opts));
//------------------------------------------------------------------
	printf("Start android Ramdisk installation. \n");

	opts.offset = 0x600000;
	opts.length = readl(0x2000FFC8);
	opts.buffer = readl(0x2000FFC0);

	ret = ft_installImage(opts,NORMAL_FS);
	

	printf("Installed Android Ramdisk image. \n");

	return ret == 0 ? 0 : 1;
	return 1;
}




//int do_insAndroidSystemImg(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
int do_insAndroidSystemImg()
{
	int ret = 0;

	nand_write_options_t opts;
	memset(&opts, 0, sizeof(opts));
//------------------------------------------------------------------
	opts.offset = 0xA00000;
	opts.length = readl(0x2000FFD8);
	opts.buffer = readl(0x2000FFD0);

	printf("Start android System Image installation. \n");

	ret = ft_installImage(opts,YAFFS_FS);

	printf("Installed Android System Image image. \n");

	return ret == 0 ? 0 : 1;
	return 1;
}




//int do_insAndroidUserImage(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
int do_insAndroidUserImage()
{
	int ret = 0;

	nand_write_options_t opts;
	memset(&opts, 0, sizeof(opts));
//------------------------------------------------------------------
	opts.offset = 0x9000000;
	opts.length = readl(0x2000FFE8);
	opts.buffer = readl(0x2000FFE0);

	printf("Start android Userdata Image installation. \n");

	ret = ft_installImage(opts,YAFFS_FS);

	printf("Installed Android UserData Image image. \n");

	return ret == 0 ? 0 : 1;
	return 1;
}




//int do_insAndroid(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
int do_insAndroid()
{
	int ret = 0;

	nand_write_options_t opts;

	memset(&opts, 0, sizeof(opts));
	opts.pad = 1;
	opts.blockalign = 1;
	opts.writeoob = 0;
	opts.autoplace = 1;
//------------------------------------------------------------------
	printf("Start android installation. \n");
	
	opts.offset = 0x0;
	opts.length = readl(0x2000FFA8);
	opts.buffer = readl(0x2000FFA0);
	if(opts.length > 0x500000)	opts.quiet  = 0;
	else				opts.quiet = 1;
	
	nand_erase(&nand_info[0], opts.buffer, &opts.length);
	ret = nand_write_opts(&nand_info[0], &opts);
	printf("Writed uBoot \n");


	opts.offset = 0x200000;
	opts.length = readl(0x2000FFB8);
	opts.buffer = readl(0x2000FFB0);

	if(opts.length > 0x500000)	opts.quiet  = 0;
	else				opts.quiet = 1;
	nand_erase(&nand_info[0], opts.buffer, &opts.length);
	ret = nand_write_opts(&nand_info[0], &opts);
	printf("Writed zImaget \n");


	opts.offset = 0x600000;
	opts.length = readl(0x2000FFC8);
	opts.buffer = readl(0x2000FFC0);
	if(opts.length > 0x500000)	opts.quiet  = 0;
	else				opts.quiet = 1;


	nand_erase(&nand_info[0], opts.buffer, &opts.length);
	ret = nand_write_opts(&nand_info[0], &opts);
	printf("Writed Ramdisk Image \n");

//------------------------------------------------------------------
	udelay (1000);
	
	opts.pad = 0;
	opts.blockalign = 1;
	opts.writeoob = 1;
	opts.autoplace = 1;
	
	opts.offset = 0xA00000;
	opts.length = readl(0x2000FFD8);
	opts.buffer = readl(0x2000FFD0);
	if(opts.length > 0x500000)	opts.quiet  = 0;
	else				opts.quiet = 1;


	printf("Start Addr : %X, Size :  %X \n",opts.buffer,opts.length);

	nand_erase(&nand_info[0], opts.buffer, &opts.length);
	ret = nand_write_opts(&nand_info[0], &opts);
	printf("Writed System Image \n");

	udelay (1000);

	opts.offset = 0x9000000;
	opts.length = readl(0x2000FFE8);
	opts.buffer = readl(0x2000FFE0);
	if(opts.length > 0x500000)	opts.quiet  = 0;
	else				opts.quiet = 1;

	printf("Start Addr : %X, Size :  %X \n",opts.buffer,opts.length);

	nand_erase(&nand_info[0], opts.buffer, &opts.length);
	ret = nand_write_opts(&nand_info[0], &opts);
	printf("Writed User Data \n");

	printf("Android Installation OK. \n");


	return ret == 0 ? 0 : 1;

	return 1;
}





int do_insAndroid_download()
{

	s3c_usbd_dn_addr = 0x20008000;	/* Default Address */
	s3c_receive_done = 0;

	s3c_usbctl_init();
	s3c_usbc_activate();

	printf("Now, Waiting for Android image downloading.\n");
	printf("Press the 'USB Port => Download' button of DNW.\n");
	
	while (1) {
		if (S3C_USBD_DETECT_IRQ()) {
			s3c_udc_int_hndlr();
			S3C_USBD_CLEAR_IRQ();
		}

		if (s3c_receive_done)
			break;

		if (serial_tstc()) {
			serial_getc();
			break;
		}
	}

	/* when operation is done, usbd must be stopped */
	s3c_usb_stop();

	return 1;
}




int do_insAndroid_install_sd(char *argv[])
{
	unsigned int addr, blk, blocks;
	uint last_blkpos, bytes;
	
	addr = 0x20008000;
	
	printf("Downloading the Android image from SD.\n");

	blk = 100000;
	bytes = (uint) simple_strtoul(argv[2], NULL, 16);

	if (blk >= 0 && blk <= ofsinfo.last) {
		blocks = (int) (bytes / MOVI_BLKSIZE);

		if (bytes % MOVI_BLKSIZE > 0)
			blocks++;
	}	
	
	while (blocks > 0) {
		if (blocks < MOVI_RW_MAXBLKS) {
			movi_read((uint) addr, blk, blocks);
			blocks = 0;
		} else {
			movi_read((uint) addr, blk, MOVI_RW_MAXBLKS);
			addr += (MOVI_RW_MAXBLKS * MOVI_BLKSIZE);
			blk += MOVI_RW_MAXBLKS;
			blocks -= MOVI_RW_MAXBLKS;
		}
	}

	printf("Writting the Android image to NAND.\n");
	
	return do_insAndroid();

	return 1;
}



int do_insUbootSD()
{

	size_t total = CFG_ENV_OFFSET;
	ulong addr = PHYS_SDRAM_1;

	movi_read((uint) addr, ofsinfo.bl2, MOVI_BL2_BLKCNT);
	nand_erase(&nand_info[0], 0x0, CFG_ENV_OFFSET + CFG_ENV_SIZE);
	nand_write(&nand_info[0], 0x0, &total, (u_char *) addr);

	printf("Installing uboot image from SD.\n");

	return 1;
}



int do_installAndroidImage(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
{
	char *cmd;
	unsigned int addr, blk, blocks;
	uint last_blkpos, bytes;

	cmd = argv[1];

	if(strcmp(cmd, "download") == 0){
		return do_insAndroid_download();
	}else if (strcmp(cmd, "android") == 0) {
		return do_insAndroid();
	}else if(strcmp(cmd, "uboot") == 0) {
		return do_insAndroidUboot();
	}else if(strcmp(cmd, "zimage") == 0) {
		return do_insAndroidzImage();
	}else if(strcmp(cmd, "ramdisk") == 0) {
		return do_insAndroidRamdisk();
	}else if(strcmp(cmd, "system") == 0) {
		return do_insAndroidSystemImg();
	}else if(strcmp(cmd, "userdata") == 0) {
		return do_insAndroidUserImage();
	}else if(strcmp(cmd, "setenv_bbp") == 0) {
		setenv("bootcmd", BOOTCOMMAND_ANDROID_BBP);
	}else if(strcmp(cmd, "setenv_normal") == 0) {
		setenv("bootcmd", BOOTCOMMAND_ANDROID_NR);
	}else if(strcmp(cmd, "android_sd") == 0){
		return do_insAndroid_install_sd(argv);
	}else if(strcmp(cmd, "uboot_sd") == 0){
		return do_insUbootSD();
	}
	
	
	return 1;
}



U_BOOT_CMD(
	insnand, 1, 0, do_insnand,
	"insnand - install SD/MMC bootloader image to NAND flash\n",
	": install SD/MMC bootloader to NAND flash\n"
);




U_BOOT_CMD(
	insAndroid,	3,	0,	do_installAndroidImage,
	"insAndroid\t- Android sub-system\n",
	"download - Download Android images (uboot/zimage/ramdisk/system/userdata)\n"
	"insAndroid android - Install all images for Android.\n"
	"insAndroid uboot - Install u-boot image\n"
	"insAndroid zimage - Install zimage image\n"
	"insAndroid ramdisk - Install ramdisk image\n"
	"insAndroid system - Install system image\n"
	"insAndroid userdata - Install userdata image\n"
	"insAndroid setenv_bbp - Set the boot command (Adaptted bad block processing)\n"	
	"insAndroid setenv_normal - Set the boot command \n"
	"insAndroid android_sd {bytes} - Download Android images from SD \n"
	"insAndroid uboot_sd - Install uboot image from SD \n"
);
#endif

